Attribute VB_Name = "groweather"
'Unit Constants

Public Const FAHRENHEIT = 0
Public Const CELSIUS = 1
Public Const INCHES = 0
Public Const MM = 1
Public Const MB = 2
Public Const HECTO_PASCAL = 3
Public Const MPH = 0
Public Const KNOTS = 1
Public Const KPH = 2
Public Const METERS_PER_SECOND = 3
 
Public Const NOT_SET = -100
Public Const COM_ERROR = -101
Public Const MEMORY_ERROR = -102
Public Const COM_OPEN = -103



Type SensorImage
    newPtr As Integer
    airTemp As Single
    soilTemp As Single
    humidity As Integer
    barometer As Single
    windSpeed As Integer
    windDirection As Integer
    totalRain As Single
	rainRate As Single
	solarRad As Integer
	totalWindRun As Single
    totalET As Single
    totalDegreeDays As Single
	totalEnergy As Single
	alarmBits As Long
	statusByte As Long
	leafByte As Integer
End Type


Type DateTime
 month As Integer
 day As Integer
 hour As Integer
 min As Integer
End Type

Type TimeHourMin
 hour As Integer
 min As Integer
End Type


Type Options
 UseDialogBox As Byte
 ClearArchive As Byte
 WriteToFile As Byte
 AppendToFile As Byte
End Type

Type AutoClearOptions
 Temp As Byte
 Wind As Byte
 DailyET As Byte
 Rain As Byte
 DegreeDay As Byte
 SolarEnergy As Byte
 Humidity As Byte
End Type




Type WeatherUnits
 TempUnit As Byte
 RainUnit As Byte
 BaromUnit As Byte
 WindUnit As Byte
End Type


Type WeatherRecord
   month As Integer
   day As Integer
   hour As Integer
   min As Integer
   hiAirTemp As Single
   lowAirTemp As Single
   soilTemp As Single
   leafWetness As Integer
   solarRad As Integer
   solarEnergy As Single
   AirTemp As Single
   barometer As Single
   Humidity As Integer
   degreeDays As Integer
   et As Single
   dewPoint As Single
   tempHumIndex As Single
   rain As Single
   rainRate As Single
   windSpeed As Integer
   highWindSpeed As Integer
   windChill As Single
   windDirection As Integer
   windRun As Integer
End Type



'Initialization Functions
Declare Function OpenCommPortG Lib "groweather.dll" (ByVal comPort As Integer, ByVal baudRate As Long, ByVal linkRev As Byte) As Integer
Declare Function OpenCommPortStrG Lib "groweather.dll" (ByVal comPort As String, ByVal baudRate As Long, ByVal linkRev As Byte) As Integer
Declare Function CloseCommPortG Lib "groweather.dll" () As Integer
Declare Function SetUnitsG Lib "groweather.dll" (structUnit As WeatherUnits) As Integer
Declare Function GetUnitsG Lib "groweather.dll" (structUnit As WeatherUnits) As Integer
Declare Function SetupRainCollectorG Lib "groweather.dll" (ByVal RainClickInc As Single, ByVal NativeUnit As Integer, ByVal DisplayUnit As Integer) As Integer


'Low Level Calls
Declare Function GetSerialCharG Lib "groweather.dll" () As Integer
Declare Function PutSerialStringG Lib "groweather.dll" (ByVal stringtoport As String) As Integer
Declare Function PutSerialCharG Lib "groweather.dll" (ByVal c As Byte) As Integer



'General Functions
Declare Function GetDllVersionG Lib "groweather.dll" () As Single
Declare Function GetPeriodG Lib "groweather.dll" () As Single
Declare Function ResetPeriodG Lib "groweather.dll" () As Integer
Declare Function GetRainRateG Lib "groweather.dll" () As Single
Declare Function GetBarometerG Lib "groweather.dll" () As Single
Declare Function GetLeafWetnessG Lib "groweather.dll" () As Integer
Declare Function GetLWEG Lib "groweather.dll" () As Integer
Declare Function GetPrimaryPowerStatusG Lib "groweather.dll" () As Integer
Declare Function GetBatteryPowerStatusG Lib "groweather.dll" () As Integer
Declare Function GetBarometerTrendG Lib "groweather.dll" () As Integer
Declare Function SetLWEG Lib "groweather.dll" (ByVal SetSensorType As Integer) As Integer
Declare Function GetAirTempG Lib "groweather.dll" () As Single
Declare Function GetTHIG Lib "groweather.dll" () As Single
Declare Function GetSoilTempG Lib "groweather.dll" () As Single
Declare Function GetHumidityG Lib "groweather.dll" () As Integer
Declare Function GetWindDirectionG Lib "groweather.dll" () As Integer
Declare Function GetModelNoG Lib "groweather.dll" () As Integer
Declare Function GetWindSpeedG Lib "groweather.dll" () As Single
Declare Function GetStationTimeG Lib "groweather.dll" (DateTimeStation As DateTime) As Integer
Declare Function SetStationTimeG Lib "groweather.dll" (DateTimeStation As DateTime) As Integer
Declare Function GetTotalRainG Lib "groweather.dll" () As Single
Declare Function GetDailyRainG Lib "groweather.dll" () As Single
Declare Function GetYearlyRainG Lib "groweather.dll" () As Single
Declare Function SetCommTimeoutValG Lib "groweather.dll" (ByVal ReadTimeout As Integer, ByVal WriteTimeout As Integer) As Integer
Declare Function SendSensorImagesG Lib "groweather.dll" (ByVal n As Integer) As Integer
Declare Function GetSensorImageG Lib "groweather.dll" (sI As SensorImage) As Integer
Declare Function StopSendingImagesG Lib "groweather.dll" () As Integer
Declare Function PutBarometerG Lib "groweather.dll" (ByVal Barom As Single) As Integer
Declare Function GetCurrentSolarRadG Lib "groweather.dll" () As Integer
Declare Function GetDailySolarRadG Lib "groweather.dll" () As Single
Declare Function GetSolarAvgEnergyG Lib "groweather.dll" () As Single
Declare Function GetDewPtG Lib "groweather.dll" () As Single
Declare Function GetWindChillG Lib "groweather.dll" () As Single
Declare Function GetTotalWindRunG Lib "groweather.dll" () As Single
Declare Function GetDailyWindRunG Lib "groweather.dll" () As Single
Declare Function GetAvgWindRunG Lib "groweather.dll" () As Single
Declare Function GetTotalETG Lib "groweather.dll" () As Single
Declare Function GetDailyETG Lib "groweather.dll" () As Single
Declare Function GetAvgETG Lib "groweather.dll" () As Single
Declare Function GetDegDayBaseTempG Lib "groweather.dll" () As Single
Declare Function GetDegDayMaxTempG Lib "groweather.dll" () As Single
Declare Function SetDegDayBaseTempG Lib "groweather.dll" (ByVal baseTemp As Single) As Single
Declare Function SetDegDayMaxTempG Lib "groweather.dll" (ByVal maxTemp As Single) As Single



'Alarms
Declare Function GetAirHiTempAlarmG Lib "groweather.dll" () As Single
Declare Function SetAirHiTempAlarmG Lib "groweather.dll" (ByVal hiAlarm As Single) As Integer
Declare Function GetAirLowTempAlarmG Lib "groweather.dll" () As Single
Declare Function SetAirLowTempAlarmG Lib "groweather.dll" (ByVal lowAlarm As Single) As Integer
Declare Function GetSoilHiTempAlarmG Lib "groweather.dll" () As Single
Declare Function SetSoilHiTempAlarmG Lib "groweather.dll" (ByVal hiAlarm As Single) As Integer
Declare Function GetSoilLowTempAlarmG Lib "groweather.dll" () As Single
Declare Function SetSoilLowTempAlarmG Lib "groweather.dll" (ByVal lowAlarm As Single) As Integer
Declare Function GetBarAlarmG Lib "groweather.dll" () As Single
Declare Function SetBarAlarmG Lib "groweather.dll" (ByVal barAlarm As Single) As Integer
Declare Function GetTimeAlarmG Lib "groweather.dll" (TimeAlarm As TimeHourMin) As Integer
Declare Function PutTimeAlarmG Lib "groweather.dll" (TimeAlarm As TimeHourMin) As Integer
Declare Function SetDewPtAlarmG Lib "groweather.dll" (ByVal TurnOn As Integer) As Integer
Declare Function GetDewPtAlarmG Lib "groweather.dll" () As Integer
Declare Function GetWindHiSpeedAlarmG Lib "groweather.dll" () As Integer
Declare Function SetWindHiSpeedAlarmG Lib "groweather.dll" (ByVal HiSpeed As Integer) As Integer
Declare Function GetLowWindChillAlarmG Lib "groweather.dll" () As Single
Declare Function SetLowWindChillAlarmG Lib "groweather.dll" (ByVal lowAlarm As Single) As Integer
Declare Function GetLowOutsideHumAlarmG Lib "groweather.dll" () As Integer
Declare Function GetHiOutsideHumAlarmG Lib "groweather.dll" () As Integer
Declare Function SetLowOutsideHumAlarmG Lib "groweather.dll" (ByVal lowAlarm As Integer) As Integer
Declare Function SetHiOutsideHumAlarmG Lib "groweather.dll" (ByVal hiAlarm As Integer) As Integer
Declare Function GetDegDayTotalAlarmG Lib "groweather.dll" () As Long
Declare Function SetDegDayTotalAlarmG Lib "groweather.dll" (ByVal hiAlarm As Single) As Integer
Declare Function GetHiTHIAlarmG Lib "groweather.dll" () As Single
Declare Function SetHiTHIAlarmG Lib "groweather.dll" (ByVal hiAlarm As Single) As Integer
Declare Function GetDailyETAlarmG Lib "groweather.dll" () As Single
Declare Function SetDailyETAlarmG Lib "groweather.dll" (ByVal hiAlarm As Single) As Integer
Declare Function GetTotalETAlarmG Lib "groweather.dll" () As Single
Declare Function SetTotalETAlarmG Lib "groweather.dll" (ByVal hiAlarm As Single) As Integer



'Hi's Lows
Declare Function GetHiLowTimesAirTempG Lib "groweather.dll" (DateTimeHiAirTemp As DateTime, DateTimeLowAirTemp As DateTime) As Integer
Declare Function GetHiLowTimesOutHumG Lib "groweather.dll" (DateTimeHiOutHum As DateTime, DateTimeLowOutHum As DateTime) As Integer
Declare Function GetLowTimesWindChillG Lib "groweather.dll" (DateTimeLowWindChill As DateTime) As Integer
Declare Function GetHiTimesTHIG Lib "groweather.dll" (DateTimeHiTHI As DateTime) As Integer
Declare Function GetHiTimesWindSpeedG Lib "groweather.dll" (DateTimeHiWindSpeed As DateTime) As Integer
Declare Function GetHiTimesRainRateG Lib "groweather.dll" (DateTimeHiRainRate As DateTime) As Integer
Declare Function GetHiAirTempG Lib "groweather.dll" () As Single
Declare Function GetLowAirTempG Lib "groweather.dll" () As Single
Declare Function GetHiSoilTempG Lib "groweather.dll" () As Single
Declare Function GetLowSoilTempG Lib "groweather.dll" () As Single
Declare Function GetHiOutsideHumG Lib "groweather.dll" () As Integer
Declare Function GetLowOutsideHumG Lib "groweather.dll" () As Integer
Declare Function GetLowWindChillG Lib "groweather.dll" () As Single
Declare Function GetHiWindSpeedG Lib "groweather.dll" () As Single
Declare Function GetHiTHIG Lib "groweather.dll" () As Single
Declare Function GetHiRainRateG Lib "groweather.dll" () As Single




'Calibration
Declare Function GetWindCalG Lib "groweather.dll" () As Single
Declare Function SetWindCalG Lib "groweather.dll" (ByVal Windcal As Integer) As Integer
Declare Function GetAirTempCalG Lib "groweather.dll" () As Single
Declare Function PutAirTempCalG Lib "groweather.dll" (ByVal calNumber As Single) As Integer
Declare Function GetSoilTempCalG Lib "groweather.dll" () As Single
Declare Function PutSoilTempCalG Lib "groweather.dll" (ByVal calNumber As Single) As Integer
Declare Function GetHumCalG Lib "groweather.dll" () As Integer
Declare Function PutHumCalG Lib "groweather.dll" (ByVal calNumber As Integer) As Integer
Declare Function PutBarometerCalG Lib "groweather.dll" (ByVal calNumber As Single) As Integer
Declare Function GetBarometerCalG Lib "groweather.dll" () As Single
Declare Function GetRainCalG Lib "groweather.dll" () As Integer
Declare Function PutRainCalG Lib "groweather.dll" (ByVal calNumber As Integer) As Integer





'Download Functions
Declare Function GetArchiveIntervalG Lib "groweather.dll" () As Integer
Declare Function SetArchiveIntervalG Lib "groweather.dll" (ByVal intervalCode As Integer) As Integer
Declare Function DownloadG Lib "groweather.dll" (DnloadOptions As Options, ByVal DnloadFileName As String) As Integer
Declare Function GetArchiveRecordG Lib "groweather.dll" (arcRecord As WeatherRecord, ByVal RecNo As Integer) As Integer
Declare Function GetArchiveRecordByAddrG Lib "groweather.dll" (arcRecord As WeatherRecord, ByVal ptr As Integer) As Integer
Declare Function GetArchRecordNoByDate Lib "groweather.dll" (recDate As DateTime) As Integer
Declare Function GetNumberOfArchiveRecords Lib "groweather.dll" () As Integer


'Clear Functions
Declare Function ClearHiAirTemperature Lib "groweather.dll" () As Integer
Declare Function ClearLowAirTemperature Lib "groweather.dll" () As Integer
Declare Function ClearHiSoilTemperature Lib "groweather.dll" () As Integer
Declare Function ClearLowSoilTemperature Lib "groweather.dll" () As Integer
Declare Function ClearHiOutsideHumidity Lib "groweather.dll" () As Integer
Declare Function ClearLowOutsideHumidity Lib "groweather.dll" () As Integer
Declare Function ClearHiWindSpeed Lib "groweather.dll" () As Integer
Declare Function ClearLowChill Lib "groweather.dll" () As Integer
Declare Function ClearDailyRain Lib "groweather.dll" () As Integer
Declare Function ClearYearlyRain Lib "groweather.dll" () As Integer
Declare Function PutTotalRain Lib "groweather.dll" (ByVal newValue As Single) As Integer
Declare Function GetAutoClear Lib "groweather.dll" (Opt As AutoClearOptions, TimeAutoClear As TimeHourMin) As Integer
Declare Function PutAutoClear Lib "groweather.dll" (Opt As AutoClearOptions, TimeAutoClear As TimeHourMin) As Integer





