/**********************************************************************/
#define DllAccess extern "C"  __declspec( dllexport )

//Unit Constants
#define INCHES                   0
#define MM                       1
#define MB                       2
#define HECTO_PASCAL             3
#define FAHRENHEIT               0
#define CELSIUS                  1
#define MPH                      0
#define KNOTS                    1
#define KPH                      2
#define METERS_PER_SECOND        3
#define KM                       1
#define MILES                    0


//Return constants
#define NOT_SET         (-100)
#define COM_ERROR       (-101)
#define MEMORY_ERROR    (-102)
#define COM_OPEN        (-103)



struct SensorImage
{   
   short int newPtr;
   float airTemp;          
   float soilTemp;          
   short int humidity;
   float barometer;
   short int windSpeed;
   short int windDirection;
   float totalRain;                          
   float rainRate;
   short int solarRad;
   float totalWindRun;
   float totalEt;                                 
   float totalDegreeDays;                    
   float totalEnergy; 
   long alarmBits;                    
   char statusByte;
   short int leafByte;
}; 


struct DateTime
{   
 short int year;
 short int month;
 short int day;
 short int hour;
 short int min;
}; 

struct TimeHourMin
{
 short int hour;
 short int min;
};

struct Options
{   
 char UseDialogBox;
 char ClearArchive;
 char WriteToFile;
 char AppendToFile;
}; 



struct AutoClearOptions
{
 char Temp;
 char Wind;
 char DailyET;
 char Rain;
 char DegreeDay;
 char SolarEnergy;
 char Humidity;
};

struct Units
{   
 char TempUnit;
 char RainUnit;
 char BaromUnit;
 char WindUnit;
}; 



struct   WeatherRecord
{
   short int month,day,hour,min;
   float hiAirTemp;
   float lowAirTemp;
   float soilTemp;
   short int leafWetness;
   short int solarRad ;
   float solarEnergy ;
   float AirTemp;
   float barometer;
   short int Humidity;
   short int degreeDays;
   float et;
   float dewPoint;
   float tempHumIndex;
   float rain;
   float rainRate;
   short int windSpeed;
   short int highWindSpeed;
   float windChill;
   short int windDirection;
   short int windRun;
};


/**********************************************************************/
//Initialization Functions


DllAccess short int _stdcall OpenCommPortG (short int comPort, int baudRate, char linkRev);
DllAccess short int _stdcall OpenCommPortStrG (char* comPort, int baudRate, char linkRev);
DllAccess short int _stdcall CloseCommPortG ();
DllAccess short int _stdcall SetUnitsG(Units* WeatherUnits);
DllAccess short int _stdcall GetUnitsG(Units* WeatherUnits);
DllAccess short int _stdcall SetupRainCollectorG(float RainClickInc,short int NativeUnit, short int DisplayUnit);


//Lowlevel Functions
DllAccess short int _stdcall GetSerialCharG ();
DllAccess short int _stdcall PutSerialStringG (const char *stringtoport);
DllAccess short int _stdcall PutSerialCharG (unsigned char c);


//General Functions
DllAccess float _stdcall GetDllVersionG();
DllAccess float _stdcall GetPeriodG ();  
DllAccess short int  _stdcall  ResetPeriodG() ;
DllAccess float _stdcall GetRainRateG ();  
DllAccess float _stdcall GetBarometerG ();  
DllAccess short int _stdcall GetLeafWetnessG(); 
DllAccess short int _stdcall GetLWEG();     
DllAccess short int _stdcall GetPrimaryPowerStatusG();  
DllAccess short int _stdcall GetBatteryPowerStatusG();   
DllAccess short int _stdcall GetBarometerTrendG();     
DllAccess short int _stdcall SetLWEG(short int SetSensorType);  
DllAccess float _stdcall GetAirTempG ();    
DllAccess float _stdcall GetTHIG ();       
DllAccess float _stdcall GetSoilTempG ();   
DllAccess short int _stdcall GetHumidityG (); 
DllAccess short int _stdcall GetWindDirectionG ();  
DllAccess short int _stdcall GetModelNoG(); 
DllAccess float _stdcall GetWindSpeedG();  
DllAccess short int _stdcall  GetStationTimeG (DateTime* DateTimeStation);  
DllAccess short int _stdcall  SetStationTimeG (DateTime* DateTimeStation);  
DllAccess float _stdcall GetTotalRainG (); 
DllAccess float _stdcall GetDailyRainG (); 
DllAccess float _stdcall GetYearlyRainG (); 
DllAccess short int _stdcall SetCommTimeoutValG(short int ReadTotalTimeout, short int WriteTotalTimeout); 
DllAccess short int _stdcall SendSensorImagesG (short int n); 
DllAccess short int _stdcall GetSensorImageG (SensorImage* sI); 
DllAccess short int _stdcall StopSendingImagesG ();
DllAccess short int _stdcall PutBarometerG(float Barom);
DllAccess short int  _stdcall GetCurrentSolarRadG();
DllAccess float  _stdcall GetDailySolarRadG();
DllAccess float _stdcall GetSolarAvgEnergyG();
DllAccess float _stdcall GetDewPointG();
DllAccess float _stdcall GetWindChillG();
DllAccess float  _stdcall GetTotalWindRunG();
DllAccess float  _stdcall GetDailyWindRunG();
DllAccess float _stdcall GetAvgWindRunG();
DllAccess float  _stdcall GetTotalETG();
DllAccess float  _stdcall GetDailyETG();
DllAccess float _stdcall GetAvgETG();
DllAccess float  _stdcall GetTotalDegDaysG();
DllAccess float  _stdcall GetDailyDegDaysG();
DllAccess float _stdcall GetAvgDegDaysG();
DllAccess float _stdcall GetDegDayBaseTempG();    
DllAccess float _stdcall GetDegDayMaxTempG();    
DllAccess short int _stdcall SetDegDayBaseTemp(float baseTemp); 
DllAccess short int _stdcall SetDegDayMaxTemp(float maxTemp); 

//Alarm Functions
DllAccess float _stdcall  GetAirHiTempAlarmG();        
DllAccess short int _stdcall  SetAirHiTempAlarmG(float hiAlarm);  
DllAccess float _stdcall  GetAirLowTempAlarmG();  
DllAccess short int _stdcall  SetAirLowTempAlarmG(float lowAlarm);  
DllAccess float _stdcall  GetSoilHiTempAlarmG();  
DllAccess short int _stdcall  SetSoilHiTempAlarmG(float hiAlarm);  
DllAccess float _stdcall  GetSoilLowTempAlarmG();  
DllAccess short int _stdcall  SetSoilLowTempAlarmG(float lowAlarm);  
DllAccess float _stdcall GetBarAlarmG ();  
DllAccess short int _stdcall SetBarAlarmG(float barAlarm);  
DllAccess short int _stdcall GetTimeAlarmG (TimeHourMin* TimeAlarm);  
DllAccess short int _stdcall PutTimeAlarmG (TimeHourMin* TimeAlarm);  
DllAccess short int _stdcall SetDewPtAlarmG (short int TurnOn);  
DllAccess short int _stdcall GetDewPtAlarmG() ;  
DllAccess short int _stdcall GetWindHiSpeedAlarmG ();  
DllAccess short int _stdcall SetWindHiSpeedAlarmG (short int HiSpeed);  
DllAccess float _stdcall GetLowWindChillAlarmG ();  
DllAccess short int _stdcall SetLowWindChillAlarmG (float lowAlarm);  
DllAccess short int _stdcall GetLowOutsideHumAlarmG (); 
DllAccess short int _stdcall GetHiOutsideHumAlarmG ();  
DllAccess short int _stdcall SetLowOutsideHumAlarmG (short int lowAlarm); 
DllAccess short int _stdcall SetHiOutsideHumAlarmG (short int hiAlarm);  
DllAccess int _stdcall GetDegDayTotalAlarmG (); 
DllAccess short int _stdcall SetDegDayTotalAlarmG (float hiAlarm);  
DllAccess float _stdcall GetHiTHIAlarmG ();               
DllAccess short int _stdcall SetHiTHIAlarmG (float hiAlarm);  
DllAccess float _stdcall GetDailyETAlarmG ();               
DllAccess short int _stdcall SetDailyETAlarmG (float hiAlarm); 
DllAccess float _stdcall GetTotalETAlarmG ();               
DllAccess short int _stdcall SetTotalETAlarmG (float hiAlarm);  


//HiLow Functions
DllAccess short int _stdcall GetHiLowTimesAirTempG (DateTime* DateTimeHiAirTemp,DateTime* DateTimeLowAirTemp);
DllAccess short int _stdcall GetHiLowTimesOutHumG (DateTime* DateTimeHiOutHum,DateTime* DateTimeLowOutHum);
DllAccess short int _stdcall GetLowTimesWindChillG (DateTime* DateTimeLowWindChill);
DllAccess short int _stdcall GetHiTimesTHIG (DateTime* DateTimeHiTHI);
DllAccess short int _stdcall GetHiTimesWindSpeedG (DateTime* DateTimeHiWindSpeed);
DllAccess short int _stdcall GetHiTimesRainRateG (DateTime* DateTimeHiRainRate);
DllAccess float _stdcall GetHiAirTempG(); 
DllAccess float _stdcall GetLowAirTempG(); 
DllAccess float _stdcall GetHiSoilTempG(); 
DllAccess float _stdcall GetLowSoilTempG(); 
DllAccess short int _stdcall GetHiOutsideHumG(); 
DllAccess short int _stdcall GetLowOutsideHumG(); 
DllAccess float _stdcall GetLowWindChillG(); 
DllAccess float  _stdcall GetHiWindSpeedG();
DllAccess float _stdcall GetHiTHIG(); 
DllAccess float _stdcall GetHiRainRateG ();


//Calibrate Functions

DllAccess float _stdcall  GetWindCalG();   
DllAccess short int _stdcall  SetWindCalG (short int Windcal); 
DllAccess float _stdcall  GetAirTempCalG();  
DllAccess short int _stdcall  PutAirTempCalG (float calNumber); 
DllAccess float _stdcall  GetSoilTempCalG ();
DllAccess short int _stdcall  PutSoilTempCalG (float calNumber);
DllAccess short int _stdcall  GetHumCalG(); 
DllAccess short int _stdcall PutHumCalG (short int calNumber); 
DllAccess short int _stdcall PutBarometerCalG (float calNumber); 
DllAccess float _stdcall GetBarometerCalG();   
DllAccess  short int _stdcall  GetRainCalG ();
DllAccess short int _stdcall  PutRainCalG (short int calNumber);

//Download functions
DllAccess short int _stdcall GetArchiveIntervalG ();
DllAccess short int _stdcall SetArchiveIntervalG (short int intervalCode);
DllAccess short int _stdcall DownloadG(Options* DnloadOptions, char* DnloadFileName);
DllAccess short int _stdcall GetArchiveRecordG(WeatherRecord* newRecord, short int i);
DllAccess short int _stdcall GetArchiveRecordByAddrG(WeatherRecord* newRecord, short int ptr);
DllAccess short int _stdcall GetArchRecordNoByDateG(DateTime* recDate);
DllAccess short int _stdcall GetNumberOfArchiveRecordsG();

//Clear functions
DllAccess short int _stdcall ClearHiAirTemperatureG();
DllAccess short int _stdcall ClearLowAirTemperatureG();
DllAccess short int _stdcall ClearHiSoilTemperatureG();
DllAccess short int _stdcall ClearLowSoilTemperatureG();
DllAccess short int _stdcall ClearHiOutsideHumidityG();
DllAccess short int _stdcall ClearLowOutsideHumidityG();
DllAccess short int _stdcall ClearHiWindSpeedG();
DllAccess short int _stdcall ClearLowChillG();
DllAccess short int _stdcall ClearDailyRainG();
DllAccess short int _stdcall ClearYearlyRainG();
DllAccess short int _stdcall PutTotalRainG (float newValue);
DllAccess short int _stdcall GetAutoClearG (AutoClearOptions* Opt, TimeHourMin* TimeAutoClear);
DllAccess short int _stdcall PutAutoClearG(AutoClearOptions* Opt, TimeHourMin* TimeAutoClear);




/**********************************************************************/
