Attribute VB_Name = "weatherlink"
'Unit Constants

Public Const FAHRENHEIT = 0
Public Const CELSIUS = 1
Public Const INCHES = 0
Public Const MM = 1
Public Const MB = 2
Public Const HECTO_PASCAL = 3
Public Const MPH = 0
Public Const KNOTS = 1
Public Const KPH = 2
Public Const METERS_PER_SECOND = 3
 
Public Const NOT_SET = -100
Public Const COM_ERROR = -101
Public Const MEMORY_ERROR = -102
Public Const COM_OPEN = -103




Type SensorImage
    outsideTemp As Single
    insideTemp As Single
    insideHum As Integer
    outsideHum As Integer
    barometer As Single
    windSpeed As Integer
    windDirection As Integer
    totalRain As Single
End Type


Type DateTime
 month As Integer
 day As Integer
 hour As Integer
 min As Integer
End Type

Type TimeHourMin
 hour As Integer
 min As Integer
End Type


Type Options
 UseDialogBox As Byte
 ClearArchive As Byte
 WriteToFile As Byte
 AppendToFile As Byte
End Type

Type WeatherUnits
 TempUnit As Byte
 RainUnit As Byte
 BaromUnit As Byte
 WindUnit As Byte
End Type


Type WeatherRecord
   month As Integer
   day As Integer
   hour As Integer
   min As Integer
   hiOutsideTemp As Single
   lowOutsideTemp As Single
   insideTemp As Single
   outsideTemp As Single
   barometer As Single
   insideHum As Integer
   outsideHum As Integer
   dewPoint As Single
   rain As Single
   windSpeed As Integer
   hiWindSpeed As Integer
   windChill As Single
   windDirection As Integer
End Type



'Initialization Functions
Declare Function OpenCommPort Lib "weatherlink.dll" (ByVal comPort As Integer, ByVal baudRate As Long) As Integer
Declare Function OpenCommPortStr Lib "weatherlink.dll" (ByVal comPort As String, ByVal baudRate As Long) As Integer
Declare Function CloseCommPort Lib "weatherlink.dll" () As Integer
Declare Function SetUnits Lib "weatherlink.dll" (structUnit As WeatherUnits) As Integer
Declare Function GetUnits Lib "weatherlink.dll" (structUnit As WeatherUnits) As Integer
Declare Function SetupRainCollector Lib "weatherlink.dll" (ByVal RainClickInc As Single, ByVal NativeUnit As Integer, ByVal DisplayUnit As Integer) As Integer


'Low Level Calls
Declare Function GetSerialChar Lib "weatherlink.dll" () As Integer
Declare Function PutSerialChar Lib "weatherlink.dll" (ByVal s1 As Byte) As Integer
Declare Function PutSerialStr Lib "weatherlink.dll" (ByVal s1 As String) As Integer

'General Functions
Declare Function GetDllVersion Lib "weatherlink.dll" () As Single
Declare Function GetBarometerTrend Lib "weatherlink.dll" () As Integer
Declare Function GetBarometer Lib "weatherlink.dll" () As Single
Declare Function GetOutsideTemp Lib "weatherlink.dll" () As Single
Declare Function GetDewPt Lib "weatherlink.dll" () As Single
Declare Function GetWindChill Lib "weatherlink.dll" () As Single
Declare Function GetInsideTemp Lib "weatherlink.dll" () As Single
Declare Function GetInsideHumidity Lib "weatherlink.dll" () As Integer
Declare Function GetOutsideHumidity Lib "weatherlink.dll" () As Integer
Declare Function GetModelNo Lib "weatherlink.dll" () As Integer
Declare Function GetTotalRain Lib "weatherlink.dll" () As Single
Declare Function GetDailyRain Lib "weatherlink.dll" () As Single
Declare Function GetYearlyRain Lib "weatherlink.dll" () As Single
Declare Function SetCommTimeoutVal Lib "weatherlink.dll" (ByVal ReadTimeout As Integer, ByVal WriteTimeout As Integer) As Integer
Declare Function SendSensorImages Lib "weatherlink.dll" (ByVal n As Integer) As Integer
Declare Function GetSensorImage Lib "weatherlink.dll" (sI As SensorImage) As Integer
Declare Function StopSendingImages Lib "weatherlink.dll" () As Integer
Declare Function PutBarometer Lib "weatherlink.dll" (ByVal Barom As Single) As Integer
Declare Function GetWindSpeed Lib "weatherlink.dll" () As Single
Declare Function GetWindDirection Lib "weatherlink.dll" () As Integer
Declare Function GetStationTime Lib "weatherlink.dll" (DateTimeStation As DateTime) As Integer
Declare Function SetStationTime Lib "weatherlink.dll" (DateTimeStation As DateTime) As Integer
Declare Function SetArchiveInterval Lib "weatherlink.dll" (ByVal intervalCode As Integer) As Integer
Declare Function GetArchiveInterval Lib "weatherlink.dll" () As Integer
Declare Function PutTotalRain Lib "weatherlink.dll" (ByVal TotalRain As Single) As Integer

'Alarms
Declare Function GetTimeAlarm Lib "weatherlink.dll" (TimeAlarm As TimeHourMin) As Integer
Declare Function PutTimeAlarm Lib "weatherlink.dll" (TimeAlarm As TimeHourMin) As Integer
Declare Function SetDewPtAlarm Lib "weatherlink.dll" (ByVal TurnOn As Integer) As Integer
Declare Function GetDewPtAlarm Lib "weatherlink.dll" () As Integer
Declare Function GetInsideLowTempAlarm Lib "weatherlink.dll" () As Single
Declare Function GetInsideHiTempAlarm Lib "weatherlink.dll" () As Single
Declare Function GetOutsideLowTempAlarm Lib "weatherlink.dll" () As Single
Declare Function GetOutsideHiTempAlarm Lib "weatherlink.dll" () As Single
Declare Function PutInsideLowTempAlarm Lib "weatherlink.dll" (ByVal lowtempAlarm As Single) As Integer
Declare Function PutInsideHiTempAlarm Lib "weatherlink.dll" (ByVal hitempAlarm As Single) As Integer
Declare Function PutOutsideLowTempAlarm Lib "weatherlink.dll" (ByVal lowtempAlarm As Single) As Integer
Declare Function PutOutsideHiTempAlarm Lib "weatherlink.dll" (ByVal hitempAlarm As Single) As Integer
Declare Function GetHiWindSpeedAlarm Lib "weatherlink.dll" () As Integer
Declare Function PutHiWindSpeedAlarm Lib "weatherlink.dll" (ByVal hiAlarm As Integer) As Integer
Declare Function GetBarAlarm Lib "weatherlink.dll" () As Single
Declare Function SetBarAlarm Lib "weatherlink.dll" (ByVal barAlarm As Single) As Integer
Declare Function GetLowWindChillAlarm Lib "weatherlink.dll" () As Single
Declare Function PutLowWindChillAlarm Lib "weatherlink.dll" (ByVal lowAlarm As Single) As Integer
Declare Function GetLowInsideHumAlarm Lib "weatherlink.dll" () As Integer
Declare Function GetHiInsideHumAlarm Lib "weatherlink.dll" () As Integer
Declare Function GetLowOutsideHumAlarm Lib "weatherlink.dll" () As Integer
Declare Function GetHiOutsideHumAlarm Lib "weatherlink.dll" () As Integer
Declare Function PutLowInsideHumAlarm Lib "weatherlink.dll" (ByVal lowAlarm As Integer) As Integer
Declare Function PutHiInsideHumAlarm Lib "weatherlink.dll" (ByVal hiAlarm As Integer) As Integer
Declare Function PutLowOutsideHumAlarm Lib "weatherlink.dll" (ByVal lowAlarm As Integer) As Integer
Declare Function PutHiOutsideHumAlarm Lib "weatherlink.dll" (ByVal hiAlarm As Integer) As Integer



'Hi's Lows
Declare Function GetHiLowTimesOutTemp Lib "weatherlink.dll" (DateTimeHiOutTemp As DateTime, DateTimeLowOutTemp As DateTime) As Integer
Declare Function GetHiLowTimesInTemp Lib "weatherlink.dll" (DateTimeHiInTemp As DateTime, DateTimeLowInTemp As DateTime) As Integer
Declare Function GetHiLowTimesOutHum Lib "weatherlink.dll" (DateTimeHiOutHum As DateTime, DateTimeLowOutHum As DateTime) As Integer
Declare Function GetHiLowTimesInHum Lib "weatherlink.dll" (DateTimeHiInHum As DateTime, DateTimeLowInHum As DateTime) As Integer
Declare Function GetHiLowTimesDewPt Lib "weatherlink.dll" (DateTimeHiDewPt As DateTime, DateTimeLowDewPt As DateTime) As Integer
Declare Function GetHiInsideTemp Lib "weatherlink.dll" () As Single
Declare Function GetLowInsideTemp Lib "weatherlink.dll" () As Single
Declare Function GetHiOutsideTemp Lib "weatherlink.dll" () As Single
Declare Function GetLowOutsideTemp Lib "weatherlink.dll" () As Single
Declare Function GetHiInsideHum Lib "weatherlink.dll" () As Integer
Declare Function GetLowInsideHum Lib "weatherlink.dll" () As Integer
Declare Function GetHiOutsideHum Lib "weatherlink.dll" () As Integer
Declare Function GetLowOutsideHum Lib "weatherlink.dll" () As Integer
Declare Function GetHiDewPt Lib "weatherlink.dll" () As Single
Declare Function GetLowDewPt Lib "weatherlink.dll" () As Single
Declare Function GetLowTimesWindChill Lib "weatherlink.dll" (DateTimeLowWindChill As DateTime) As Integer
Declare Function GetLowWindChill Lib "weatherlink.dll" () As Single
Declare Function GetHiTimesWindSpeed Lib "weatherlink.dll" (DateTimeHiWindSpeed As DateTime) As Integer
Declare Function GetHiWindSpeed Lib "weatherlink.dll" () As Single


'Calibration
Declare Function GetBarometerCal Lib "weatherlink.dll" () As Single
Declare Function PutBarometerCal Lib "weatherlink.dll" (ByVal calNumber As Single) As Integer
Declare Function GetInsideTempCal Lib "weatherlink.dll" () As Single
Declare Function PutInsideTempCal Lib "weatherlink.dll" (ByVal calNumber As Single) As Integer
Declare Function GetOutsideTempCal Lib "weatherlink.dll" () As Single
Declare Function PutOutsideTempCal Lib "weatherlink.dll" (ByVal calNumber As Single) As Integer
Declare Function GetHumCal Lib "weatherlink.dll" () As Integer
Declare Function PutHumCal Lib "weatherlink.dll" (ByVal calNumber As Long) As Integer
Declare Function GetRainCal Lib "weatherlink.dll" () As Integer
Declare Function PutRainCal Lib "weatherlink.dll" (ByVal calNumber As Integer) As Integer



'Download Functions
Declare Function GetNumberOfArchiveRecords Lib "weatherlink.dll" () As Integer
Declare Function Download Lib "weatherlink.dll" (DnloadOptions As Options, ByVal DnloadFileName As String) As Integer
Declare Function GetArchiveRecord Lib "weatherlink.dll" (arcRecord As WeatherRecord, ByVal RecNo As Integer) As Integer
Declare Function GetArchRecordNoByDate Lib "weatherlink.dll" (recDate As DateTime) As Integer


'Clear Functions
Declare Function ClearHiInsideTemperature Lib "weatherlink.dll" () As Integer
Declare Function ClearLowInsideTemperature Lib "weatherlink.dll" () As Integer
Declare Function ClearHiOutsideTemperature Lib "weatherlink.dll" () As Integer
Declare Function ClearLowOutsideTemperature Lib "weatherlink.dll" () As Integer
Declare Function ClearHiOutsideHumidity Lib "weatherlink.dll" () As Integer
Declare Function ClearLowOutsideHumidity Lib "weatherlink.dll" () As Integer
Declare Function ClearHiInsideHumidity Lib "weatherlink.dll" () As Integer
Declare Function ClearLowInsideHumidity Lib "weatherlink.dll" () As Integer
Declare Function ClearHiDewPt Lib "weatherlink.dll" () As Integer
Declare Function ClearLowDewPt Lib "weatherlink.dll" () As Integer
Declare Function ClearHiWindSpeed Lib "weatherlink.dll" () As Integer
Declare Function ClearLowChill Lib "weatherlink.dll" () As Integer
Declare Function ClearDailyRain Lib "weatherlink.dll" () As Integer
Declare Function ClearYearlyRain Lib "weatherlink.dll" () As Integer




