/**********************************************************************/
#define DllAccess extern "C"  __declspec( dllexport )

//Unit Constants
#define INCHES                   0
#define MM                       1
#define MB                       2
#define HECTO_PASCAL             3
#define FAHRENHEIT               0
#define CELSIUS                  1
#define MPH                      0
#define KNOTS                    1
#define KPH                      2
#define METERS_PER_SECOND        3
#define KM                       1
#define MILES                    0


//Return constants
#define NOT_SET         (-100)
#define COM_ERROR       (-101)
#define MEMORY_ERROR    (-102)
#define COM_OPEN        (-103)


struct SensorImage
{   
   float outsideTemp;          
   float insideTemp;          
   short int insideHum;
   short int outsideHum;
   float barometer;
   short int windSpeed;
   short int windDirection;
   float totalRain;                          // Total rain on weather station.
}; 


struct DateTime
{   
 short int month;
 short int day;
 short int hour;
 short int min;
}; 

struct TimeHourMin
{
 short int hour;
 short int min;
};

struct Options
{   
 char UseDialogBox;
 char ClearArchive;
 char WriteToFile;
 char AppendToFile;
}; 

struct WeatherUnits
{   
 char TempUnit;
 char RainUnit;
 char BaromUnit;
 char WindUnit;
}; 



struct   WeatherRecord
{
   short int month,day,hour,min;
   float hiOutsideTemp;
   float lowOutsideTemp;
   float insideTemp;
   float outsideTemp;
   float barometer;
   short int insideHum;
   short int outsideHum;
   float dewPoint;
   float rain;
   short int windSpeed;
   short int hiWindSpeed;
   float windChill;
   short int windDirection;
};

/**********************************************************************/
//Initialization Functions

DllAccess short int _stdcall OpenCommPort (short int comPort, int baudRate);
DllAccess short int _stdcall OpenCommPortStr (char* comPort, int baudRate);
DllAccess short int _stdcall CloseCommPort ();
DllAccess short int _stdcall SetUnits(WeatherUnits* Units);
DllAccess short int _stdcall GetUnits(WeatherUnits* Units);
DllAccess short int _stdcall SetupRainCollector(float RainClickInc,short int NativeUnit, short int DisplayUnit);


//Lowlevel Functions
DllAccess short int _stdcall GetSerialChar ();
DllAccess short int _stdcall PutSerialStr (char *stringtoport);
DllAccess int _stdcall PutSerialChar (unsigned char c);


//General Functions
DllAccess float _stdcall GetDllVersion();
DllAccess short int _stdcall GetBarometerTrend ();
DllAccess float _stdcall GetBarometer ();
DllAccess float _stdcall GetOutsideTemp ();
DllAccess float _stdcall GetDewPt ();
DllAccess float _stdcall GetWindChill ();
DllAccess float _stdcall GetInsideTemp ();
DllAccess short int _stdcall GetInsideHumidity ();
DllAccess short int _stdcall GetOutsideHumidity ();
DllAccess short int _stdcall GetModelNo();
DllAccess float _stdcall GetTotalRain ();
DllAccess float _stdcall GetDailyRain ();
DllAccess float _stdcall GetYearlyRain ();
DllAccess short int _stdcall SetCommTimeoutVal(short int ReadTimeout, short int WriteTimeout);
DllAccess short int _stdcall SendSensorImages (short int n);
DllAccess short int _stdcall GetSensorImage (SensorImage* sI);
DllAccess short int _stdcall StopSendingImages ();
DllAccess short int _stdcall PutBarometer(float Barom);
DllAccess float _stdcall GetWindSpeed();
DllAccess short int _stdcall GetWindDirection();
DllAccess short int _stdcall  GetStationTime (DateTime* DateTimeStation);
DllAccess short int _stdcall  SetStationTime (DateTime* DateTimeStation);
DllAccess short int _stdcall  GetArchiveInterval ();
DllAccess short int _stdcall SetArchiveInterval (short int intervalCode);
DllAccess short int _stdcall PutTotalRain (float TotalRain);

//Alarm Functions
DllAccess short int _stdcall GetTimeAlarm (TimeHourMin* TimeAlarm);
DllAccess short int _stdcall PutTimeAlarm (TimeHourMin* TimeAlarm);
DllAccess short int _stdcall SetDewPtAlarm (short int TurnOn);
DllAccess short int _stdcall GetDewPtAlarm() ;
DllAccess float _stdcall  GetInsideLowTempAlarm ();
DllAccess float _stdcall  GetInsideHiTempAlarm ();
DllAccess float _stdcall  GetOutsideLowTempAlarm ();
DllAccess float _stdcall  GetOutsideHiTempAlarm ();
DllAccess short int  _stdcall  PutInsideLowTempAlarm (float lowtempAlarm);
DllAccess short int _stdcall  PutInsideHiTempAlarm (float hitempAlarm);
DllAccess short int _stdcall  PutOutsideLowTempAlarm (float lowtempAlarm);
DllAccess short int _stdcall  PutOutsideHiTempAlarm (float hitempAlarm);
DllAccess short int _stdcall GetHiWindSpeedAlarm ();
DllAccess short int _stdcall PutHiWindSpeedAlarm (short int hiAlarm);
DllAccess float _stdcall GetBarAlarm ();
DllAccess short int _stdcall SetBarAlarm(float barAlarm);  
DllAccess float _stdcall GetLowWindChillAlarm ();
DllAccess short int _stdcall PutLowWindChillAlarm (float lowAlarm);

DllAccess short int _stdcall GetLowInsideHumAlarm ();
DllAccess short int _stdcall GetHiInsideHumAlarm ();
DllAccess short int _stdcall GetLowOutsideHumAlarm ();
DllAccess short int _stdcall GetHiOutsideHumAlarm ();

DllAccess short int _stdcall PutLowInsideHumAlarm (short int lowAlarm);
DllAccess short int _stdcall PutHiInsideHumAlarm (short int hiAlarm);
DllAccess short int _stdcall PutLowOutsideHumAlarm (short int lowAlarm);
DllAccess short int _stdcall PutHiOutsideHumAlarm (short int hiAlarm);

//HiLow Functions
DllAccess short int _stdcall GetHiLowTimesOutTemp (DateTime* DateTimeHiOutTemp,DateTime* DateTimeLowOutTemp);
DllAccess short int _stdcall GetHiLowTimesInTemp (DateTime* DateTimeHiInTemp,DateTime* DateTimeLowInTemp);
DllAccess short int _stdcall GetHiLowTimesOutHum (DateTime* DateTimeHiOutHum,DateTime* DateTimeLowOutHum);
DllAccess short int _stdcall GetHiLowTimesInHum (DateTime* DateTimeHiInHum,DateTime* DateTimeLowInHum);
DllAccess short int _stdcall GetHiLowTimesDewPt (DateTime* DateTimeHiDewPt,DateTime* DateTimeLowDewPt);
DllAccess float _stdcall GetHiInsideTemp(); 
DllAccess float _stdcall GetLowInsideTemp(); 
DllAccess float _stdcall GetHiOutsideTemp(); 
DllAccess float _stdcall GetLowOutsideTemp(); 
DllAccess short int _stdcall GetHiInsideHum(); 
DllAccess short int _stdcall GetLowInsideHum(); 
DllAccess short int _stdcall GetHiOutsideHum(); 
DllAccess short int _stdcall GetLowOutsideHum(); 
DllAccess float _stdcall GetHiDewPt(); 
DllAccess float _stdcall GetLowDewPt(); 
DllAccess short int _stdcall GetLowTimesWindChill (DateTime* DateTimeLowWindChill);
DllAccess float _stdcall GetLowWindChill(); 
DllAccess short int _stdcall GetHiTimesWindSpeed (DateTime* DateTimeHiWindSpeed);
DllAccess float  _stdcall GetHiWindSpeed();


//Calibrate Functions
DllAccess short int _stdcall PutBarometerCal (float calNumber);
DllAccess float _stdcall GetBarometerCal();
DllAccess float _stdcall  GetInsideTempCal ();
DllAccess short int _stdcall  PutInsideTempCal (float calNumber);
DllAccess float _stdcall  GetOutsideTempCal();
DllAccess short int _stdcall  PutOutsideTempCal (float calNumber);
DllAccess short int _stdcall  GetHumCal();
DllAccess short int _stdcall PutHumCal (int calNumber);
DllAccess short int _stdcall  GetRainCal();
DllAccess short int _stdcall  PutRainCal (int calNumber);

//Download functions
DllAccess short int _stdcall GetNumberOfArchiveRecords();
DllAccess short int _stdcall Download(Options* DnloadOptions, char* DnloadFileName);
DllAccess short int _stdcall GetArchiveRecord(WeatherRecord* newRecord, short int i);
DllAccess short int _stdcall GetArchRecordNoByDate(DateTime* recDate);

//Clear functions
DllAccess short int _stdcall ClearHiInsideTemperature();
DllAccess short int _stdcall ClearLowInsideTemperature();
DllAccess short int _stdcall ClearHiOutsideTemperature();
DllAccess short int _stdcall ClearLowOutsideTemperature();
DllAccess short int _stdcall ClearHiOutsideHumidity();
DllAccess short int _stdcall ClearLowOutsideHumidity();
DllAccess short int _stdcall ClearHiInsideHumidity();
DllAccess short int _stdcall ClearLowInsideHumidity();
DllAccess short int _stdcall ClearHiDewPoint();
DllAccess short int _stdcall ClearLowDewPoint();
DllAccess short int _stdcall ClearHiWindSpeed();
DllAccess short int _stdcall ClearLowChill();
DllAccess short int _stdcall ClearDailyRain();
DllAccess short int _stdcall ClearYearlyRain();



/**********************************************************************/
