#!/usr/bin/perl -w
# weather2gp.pl diverts standard input to corresponding time slots files.
# By amn, jen 19-Nov-1998 Metshovi Radio Observatory.

# $Log: weather2gp.pl,v $
# Revision 1.2  1998/11/23 09:02:31  amn
# Changed 'ls -t' into 'ls -r'; files may be edited but names have time in them.
#
# Revision 1.1  1998/11/20 14:19:06  jen
# Initial version of gnuplot plotting software.
#


use English;  # Needed because $OS_ERROR will not work otherwise.
use FileHandle;

# Channel names.
$hilo = "hi";
@ch = ("Stat", "OutT", "RelH", "Baro", "DewP", "Rain", "Leaf", "Sola", "Wind", "WDir", "Volt");

while (1) {

# Hash tables of file names and file handles per each channel.
%chfn = ();
%chfile = ();
foreach $ch (@ch) {
    my $fn = "/tmp/weather_".$hilo."_".$ch.$$;
    $chfn{$ch} = $fn;
    $chfile{$ch} = new FileHandle;
    # Open one output file per each channel.
    open $chfile{$ch}, "> $fn" or die "Cannot write to $fn ($OS_ERROR).";
    # Save all opened file names for 'rm -rf' at end of this script.
    push @allfiles, $fn;
}

# Find the latest data log file name.
$data_dir = "/local/weather/$hilo";
# List files in reverse alphabetical order to get the newest file.
# ('ls -t' may list files edited last but which were created earlier...)
$file_name = "$data_dir/".`ls -r $data_dir | head -1 `;

# Read all data from it and "demux" it into per-channel temp files.
open INFILE, "< $file_name" or die "Cannot read from $file_name ($OS_ERROR).";
while (<INFILE>)
{
    ($time, $ch, $value) = split("\t", $_, 3);
    print { $chfile{$ch} } $time, "\t", $value; 
}

foreach $ch (@ch) {
    # Close every output file per each channel.
    close $chfile{$ch} or die "Cannot close $chfn{$ch} ($OS_ERROR).";
}


# Create Gnuplot command file.
$gpf = "/tmp/set_weather_defaults$$";
open GPF, "> $gpf" or die "Cannot write to $gpf ($OS_ERROR).";
push @allfiles, $gpf;
print GPF "set xdata time\n";
print GPF "set timefmt \"%Y%m%d%H%M%S\"\n";
print GPF "set xrange [*:*]\n";
print GPF "set y2range [*:*]\n";
print GPF "set y2tics\n";
print GPF "plot \"" . $chfn{"OutT"} . "\" using 1:2 title \"" . $hilo . " OutT\" with lines, \"" . $chfn{"RelH"} . "\" using 1:2 axes x1y2 title \"" . $hilo . " RelH\" with lines, \"" . $chfn{"DewP"} . "\" using 1:2 axes x1y1 title \"" . $hilo . " DewP\" with lines\n";
#print GPF "pause 120\n";

close GPF or die "Closet test $gpf ($OS_ERROR).";

# Run Gnuplot.

system "gnuplot $gpf";

# Remove temp files.
system "rm -rf @allfiles";

} # while forever!!
